import { Disposable } from './disposable.interface';
import { OnDestroy } from '@angular/core';
import { Destructor } from './destructor.type';
import { Observable } from 'rxjs';
import { Unsubscribable, Observer } from 'rxjs';
export declare class Cycler implements Disposable, OnDestroy {
    private _destructors;
    private _transientDestructors;
    /**
     * Gets an amount of destructors currently in the queue.
     */
    getDestructorsCount(): number;
    /**
     * Disposes the Cycler causing all it's dependant managed
     * entities to be disposed. This method will be called
     * automatically by Angular if the Cycler is properly
     * provided.
     */
    dispose(): void;
    /**
     * Angular lifecycle hook implementation. Expected to be
     * called by Angular automatically. There are two ways to
     * achieve that:
     *
     * ### Option 1: provide the Cycler object:
     * ```ts
     * import { Cycler } from 'ng-cycler';
     *
     * @Component({
     *   // ...
     *   providers: [Cycler] // provide component scope-related Cycler instance
     * })
     * export class MyComponent {
     *     constructor(private readonly _cycler: Cycler){
     *         // use _cycler wherever needed
     *     }
     * }
     * ```
     * ### Option 2: extend your component from Cycler:
     *
     * ```ts
     * import { Cycler } from 'ng-cycler';
     *
     * export class MyComponent extends Cycler {
     *     // use this. to call Cycler-related methods
     * }
     * ```
     */
    ngOnDestroy(): void;
    /**
     * Adds a Destructor to the disposal queue to be called once the Cycler
     * is finalized by Angular.
     *
     * @param destructor to be added to the disposal queue
     * @returns disposable that can be used to run the destructor at any
     * time by calling `dispose` method. In case of `dispose` forced
     * manually it will be excluded from the main disposal queue.
     */
    manageDestructor(destructor: Destructor): Disposable;
    /**
     * Adds a Disposable object to the disposal queue. The logic is the same
     * as for `manageDestructor` but using a Disposable instead of Destructor.
     *
     * See `manageDestructor` for details.
     *
     * @param disposable to be added to the disposal queue
     *
     * @returns disposable that wraps the original disposable. This new Disposable
     * can be used to finalize the current one and remove it from the queue at the
     * same time.
     */
    manageDisposable(disposable: Disposable): Disposable;
    /**
     * Adds a subscription to the disposal queue. The subscription will be
     * finalized (by calling `unsubscribe` method) once the Cycler instance
     * is disposed.
     *
     * @param {Unsubscribable} subscription to be added to the disposal queue.
     *
     * @returns {Disposable} disposable object that can be used to finalize
     * the subscription manually.
     */
    manageSubscription(subscription: Unsubscribable): Disposable;
    /**
     * Adds a dependency to the disposal queue to be managed by the Cycler.
     *
     * The dependency could be one of the following:
     *
     *   * {@link Destructor} destructor - a callback will be invoked on dependency finalization;
     *   * {@link Disposable} disposable - the `dispose` method will be called on dependency finalization;
     *   * {@link Unsubscribable} subscription - the `unsubscribe` method will be called on dependency finalization;
     *
     * The dependency will be finalized on:
     *
     *   * the Cycler disposal by Angular;
     *   * manually by calling `dispose` on the result object;
     *
     * @param dependency to be added to the finalization queue.
     */
    manage(dependency: Destructor | Disposable | Unsubscribable): Disposable;
    /**
     * Adds multiple dependencies to the disposal queue.
     *
     * @param dependencies
     */
    manageAll(...dependencies: (Destructor | Disposable | Unsubscribable)[]): void;
    manageTransient(code: string, dependency: Destructor | Disposable | Unsubscribable): Disposable;
    managedSubscribe<T>(observable: Observable<T>, observerOrNext?: Partial<Observer<T>> | ((value: T) => void)): Disposable;
}
